/**
 * Create a standard way to interact with the interpreter.
 * At some later point, it might make sense to split
 * apart the components and make them programmable
 * from scheme, but for now, just provide a small and
 * quick solution to making a UI.
 */

package sbktech.jirc;

import java.io.*;
import java.awt.*;
import java.util.Hashtable;
import sbktech.tools.scm.*;

class InterpreterFrame extends Frame implements Runnable
{
  TextArea display_area;
  Button eval_button;
  Button close_button;
  PrintStream out;
  parser local_interp;
  StreamTokenizer inp;

  InterpreterFrame(String title)
  {
    super(title);
    Panel p;
    add("Center", display_area = new TextArea());
    Font fnt = new Font("Courier", Font.PLAIN, 14);
    if (fnt != null)
      {
        display_area.setFont(fnt);
      }

    add("South", p = new Panel());
    p.add(eval_button = new Button("Eval"));
    p.add(close_button = new Button("Close"));
                                // Still interact with the global
                                // environment, but this time
                                // take the input from the
                                // text area.
    resize(500, 300);
    show();

    try
      {
        PipedInputStream pin = new PipedInputStream();
        out = new PrintStream(new PipedOutputStream(pin));
        inp = new StreamTokenizer(pin);
        local_interp = new parser(inp, driver.global);
        new Thread(this).start();
      }
    catch (Exception ex)
      {}
  }
  public void run()
  {
    try
      {
        do
          {
            try
              {
                Obj ret = local_interp.eval();
                if (ret == null)
                  { display_area.appendText("nil"); }
                else
                  { display_area.appendText(ret.toString()); }
              }
            catch(SchemeError inner_ex)
              {
                display_area.appendText("Exception: " + inner_ex.getMessage());
              }
          }
        while (inp.ttype != inp.TT_EOF);
      }
    catch(Exception exc)
      { return; }
  }

  public boolean action(Event e, Object arg)
  {
    if (e.target == eval_button)
      {
        out.println(display_area.getText());
        display_area.setText("");
        return true;
      }
    else if (e.target == close_button)
      {
        try
          {
            out.close();
          }
        catch(Exception r) {}
        dispose();
        return true;
      }
    return false;
  }
}
