/**
 * Create a standard way to display messages.
 * At some later point, it might make sense to split
 * apart the components and make them programmable
 * from scheme, but for now, just provide a small and
 * quick solution to making a UI.
 */

package sbktech.jirc;

import java.awt.*;
import java.util.Hashtable;
import sbktech.tools.scm.*;

class SimpleFrame extends Frame
{
  TextArea display_area;
  TextField edit_field;
  TextField target_field;
  Procedure proc;
  MenuBar mbar;
  Panel commands_area;
  GridBagLayout gb;
  GridBagConstraints gc;
  int current_gridy;
  Hashtable commands = new Hashtable();

  SimpleFrame(String title)
  {
    super(title);
    Font fnt = new Font("Helvetica", Font.PLAIN, 14);
    add("Center", display_area = new TextArea());
    if (fnt != null)
      {
        display_area.setFont(fnt);
      }

    display_area.setEditable(false);
    Panel p = new Panel();
    BorderLayout b = new BorderLayout();
    p.setLayout(new BorderLayout());
    p.add("West", new Label("Talk to target: "));
    p.add("Center", edit_field = new TextField());
    if (fnt != null)
      {
        edit_field.setFont(fnt);
      }

    add("South", p);

    p = new Panel();
    p.setLayout(b);
    p.add("West", new Label("Current Target: "));
    p.add("Center", target_field = new TextField());
    add("North", p);

    commands_area = new Panel();
    add("West", commands_area);
    commands_area.setLayout(gb = new GridBagLayout());
    gc = new GridBagConstraints();
    gc.weightx = 1.0;
    current_gridy = 0;
    gc.fill = gc.HORIZONTAL;
    mbar = new MenuBar();
    setMenuBar(mbar);
    resize(500, 400);
  }


  String getTarget()
  {
    return(target_field.getText());
  }

  void appendDisplay(String s)
  {
    display_area.appendText(s);
  }

  void setUserHook(Procedure proc)
  {
    this.proc = proc;
  }

  void addMenu(Menu m)
  {
    mbar.add(m);
  }

  void addCommand(Button b)
  {
    gc.gridy = current_gridy++;
    gb.setConstraints(b, gc);
    commands_area.add(b);
  }

  void setMenuItemHook(MenuItem m, Procedure proc)
  {
    commands.put(m, proc);
  }

  void setCommandHook(Button b, Procedure proc)
  {
    commands.put(b, proc);
  }

  public boolean action(Event e, Object arg)
  {
    if (e.target == edit_field)
      {
        String s = edit_field.getText();
        edit_field.setText("");
        try
          {
            if ((s != null) && (s.length() > 0) && (proc != null))
              {
                Cell mwrap = new Cell(new Selfrep(s), null);
                proc.apply(mwrap, driver.global);
              }
          }
        catch (Exception ex)
          {}                    // silently ignore... FIXMEEE
        return true;
      }
    else
      {
        Procedure hook = (Procedure)(commands.get(e.target));
        if (hook != null)
          {
            try
              {
                hook.apply(null, driver.global);
              }
            catch(Exception ex)
              {}                // silently ignore.. FIXMEE
            return true;
          }
      }
    if (e.id == e.WINDOW_DESTROY)
      { dispose(); return true; }

    return false;
  }
}
