/**
 * This is the place where toplevel information
 * about an IRC connection is setup. There is one
 * ircEnv for each connection to an IRC server.
 *
 * @author $Author: kbs $
 * @revision $Revision: 1.3 $
 */

package sbktech.jirc;

import java.util.*;
import java.net.*;
import java.io.*;

import sbktech.tools.scm.*;

class ircEnv
{
  Socket ircsock;
  String irchost;
  int ircport;
  String ircnick;
  String ircuser;
  String ircpassw;
  String ircrealname;
  Hashtable handler = new Hashtable();
  Procedure defhandler;

                                // Spawn off threads to listen and
                                // send information
  RecvHandler irclistener;
  SendHandler ircsender;

  ircEnv()
  { super(); }

  void populate(String host, int port, String nick, String user, String passw,
                String realname)
  {
    ircport = port;
    ircnick = nick;
    ircuser = user;
    ircpassw = passw;
    ircsock = null;
    irchost = host;
    ircrealname = realname;
  }

  void connect()
    throws IOException
  {
    if (ircnick == null) return;
    ircsock = new Socket(irchost, ircport);

                                // Spawn a thread to listen to
                                // server inputs
    irclistener =
      new RecvHandler(this);
    irclistener.start();

                                // And similarly to handle sends to
                                // server
    ircsender =
      new SendHandler(this);
    ircsender.start();

                                // Send initial handshake sequence
    if (ircpassw != null)
      {
        ircsender.send("PASS " + ircpassw + "\r\n");
      }
    ircsender.send("NICK " + ircnick + "\r\n");
    ircsender.send
      ("USER " + ircuser + " localhost " + irchost + " :" +
       ircrealname + "\r\n");
  }

  void close()
  {
    try { ircsock.close(); }
    catch (Exception e) {}
    try { ircsender.done = true; send("QUIT\r\n"); }
    catch (Exception e) {}
  }

  void setMsgHook(String command, Procedure proc)
  {
    handler.put(command, proc);
  }

  void setDefaultMsgHook(Procedure proc)
  {
    defhandler = proc;
  }

  void send(String s)
    throws IOException
  { ircsender.send(s); }
}

class RecvHandler extends Thread
{
  DataInputStream in;
  ircEnv parent;

  RecvHandler(ircEnv ie)
    throws IOException
  {
    in = new DataInputStream
      (new BufferedInputStream
       (ie.ircsock.getInputStream()));
    parent = ie;
  }


                                // This is the place where
                                // scheme event handlers are
                                // added

  public void run()
  {
    while (true)
      {
        try
          {
            String thisLine = in.readLine();
            if (thisLine == null)
              {
                parent.close();
                return;
              }
            message m = new message(thisLine);
                                // Search for a handler, otherwise
                                // just print it out.
            //     System.out.println("Got back a "  + m);
            if (m.command != null)
              {
                Procedure h = (Procedure)(parent.handler.get(m.command));
                if ((h != null) ||
                    ((h = parent.defhandler) != null))
                  {
                                // Make a wrapper for the msg
                    Cell mwrap = new Cell(new primnode(m), null);
                    h.apply(mwrap, driver.global);
                  }
                else
                  { System.out.println(m); }
              }
          }
        catch(Exception e)
          {
            parent.close();
            return;
          }
      }
  }
}

class SendHandler extends Thread
{
  DataOutputStream out;
  ircEnv parent;
  obj_queue myqueue;
  boolean done = false;

  SendHandler(ircEnv ie)
    throws IOException
  {
    out = new DataOutputStream
      (new BufferedOutputStream
       (ie.ircsock.getOutputStream()));
    myqueue = new obj_queue(this);
    parent = ie;
  }

  void send(String s)
  {
    myqueue.do_op(obj_queue.APPEND, s);
  }

  public synchronized void run()
  {
    while (true)
      {
        try
          {
            String send_string = null;
            while(!done &&
                  ((send_string =
                   (String)myqueue.do_op(obj_queue.HEAD, null)) == null))
              {
                suspend();
              }

            if (done) return;
            //        System.out.print("sender: sending " + send_string);
            //        System.out.flush();
            out.writeBytes(send_string);
            out.flush();
          }
        catch(Exception e)
          {
            parent.close();
            return;
          }
      }
  }
}

class obj_queue
{
  obj_node data;
  obj_node head;
  obj_node tail;
  Thread parent;
  boolean also_notify = false;

  static final int HEAD = 1;
  static final int APPEND = 2;

  obj_queue(Thread p)
  {
    parent = p;
  }

  synchronized Object do_op(int command, Object arg)
  {
    switch(command)
      {
      case HEAD:
        if (head == null)
          {
            also_notify = true;
            return null;
          }
        Object ret = head.data;
        head = head.next;
        if (head == null)
          {
            tail = null;
          }
        return ret;
      case APPEND:
        if (tail == null)
          {
            tail = new obj_node(arg);
            head = tail;
            if (also_notify)
              {
                try
                  {
                    parent.resume();
                  }
                catch(Exception e)
                  {
                    System.out.println("notify warning");
                    e.printStackTrace();
                  }
                also_notify = false;
              }
          }
        else
          {
            tail.next = new obj_node(arg);
            tail = tail.next;
          }
        return arg;
      default:
        System.out.println("Warning: ignoring unknown tag " + command);
        return null;
      }
  }
}

class obj_node
{
  Object data;
  obj_node next;
  obj_node(Object d) { data = d; next = null; }
}

