;;; This is  an example IRC client with a minimum set
;;; of handlers. Modify to suit taste.


;; Utility procedures


;; irc-make-ui creates the display frame which shows the
;; actual messages, and a set of handlers and menubuttons
;; and buttons to handle common tasks.

(define irc-make-ui
  (lambda (title curr-irc-env)
    (define top (make-frame title))

    (define file-menu (make-menu "File"))

                                        ; One place to start it all
    (define connect-item
      (make-menu-item "Connect..."))
    (menu-add-item file-menu connect-item)

    (define quit-item
      (make-menu-item "Quit IRC"))
    (menu-add-item file-menu quit-item)
                                        ; swoosh capability
    (define swoosh-item
      (make-menu-item "Swoosh jirc user"))
    (menu-add-item file-menu swoosh-item)

                                        ; separator.
                                        ; This is java's kludge, not mine
    (menu-add-item file-menu (make-menu-item "-"))
    
                                        ; The real goody
    (define interp-item
      (make-menu-item "Interpreter..."))
    (menu-add-item file-menu interp-item)
    (menu-add-item file-menu (make-menu-item "-"))

    (define exit-item
      (make-menu-item "Exit"))
    (menu-add-item file-menu exit-item)

    (frame-add-menu top file-menu)

                                        ; Get info to initiate connection
    (define connect-proc
      (lambda ()
        (define query-retrieve
          (lambda (host nick user rname)
            (irc-env-set!
             curr-irc-env host 6667 nick user () rname)
            (frame-append-text
             top  (concat "Connecting to " host "...\n"))
            (irc-connect curr-irc-env)))
        (define my-query
          (make-query-box
           (quote ("IRC Server:" "Nick        :" "Username :" "Real name:"))))
        (query-box-set-hook! my-query query-retrieve)))
    (frame-set-menu-item-hook! top connect-item connect-proc)


                                        ; Swoosh user
    (define swoosh-proc
      (lambda ()
        (define query
          (make-query-box (quote ("User to swoosh:" "URL        :"))))
        (query-box-set-hook! query jirc-swoosh)))
    (frame-set-menu-item-hook! top swoosh-item swoosh-proc)

                                        ; Quit IRC
    (frame-set-menu-item-hook!
     top
     quit-item
     (lambda ()
       (irc-send curr-irc-env
                 (concat "QUIT :Leaving\r\n"))))


                                        ; plug into the interpreter

    (frame-set-menu-item-hook!
     top interp-item
     (lambda ()
       (make-interpreter-frame "Interpreter")))

                                        ; Exit / dispose interp
    (frame-set-menu-item-hook!
     top exit-item
     (lambda ()
       (frame-dispose top)))

                                        ; A few commands that
                                        ; are more easily
                                        ; accessible
    (define join-command (make-command "Join"))
    (frame-add-command top join-command)

    (frame-set-command-hook!
     top  join-command
     (lambda ()
       (irc-send curr-irc-env
                 (concat "JOIN "
                         (frame-get-current-target top)
                         "\r\n"))))

    (define leave-command (make-command "Leave"))
    (frame-add-command top leave-command)

    (frame-set-command-hook!
     top  leave-command
     (lambda ()
       (irc-send curr-irc-env
                 (concat "PART "
                         (frame-get-current-target top)
                         "\r\n"))))

    (define names-command (make-command "Names"))
    (frame-add-command top names-command)

    (frame-set-command-hook!
     top  names-command
     (lambda ()
       (irc-send curr-irc-env
                 (concat "NAMES "
                         (frame-get-current-target top)
                         "\r\n"))))

    top))

;; General utility procedures that are accessible
;; from the interpreter

(define command
  (lambda (s)
    (irc-send default-irc-env
              (concat s "\r\n"))))
(define ctcp
  (lambda (target msg)
    (irc-send default-irc-env
              (concat "PRIVMSG " target " :\001" msg "\001\r\n"))))

(define meta-command
  (lambda (s)
    (lambda (target)
      (irc-send default-irc-env
                (concat s target "\r\n")))))

;; standard set of single argument calls, except in scm

(define nick (meta-command "NICK "))
(define join (meta-command "JOIN "))
(define part (meta-command "PART "))
(define whois (meta-command "WHOIS "))


(define jirc-swoosh
  (lambda (target url)
    (irc-send default-irc-env
              (concat
               "PRIVMSG " target " :\001JIRC-SWOOSH " url "\001\r\n"))))


(define default-irc-env (make-irc-env))

(define test-ui
  (irc-make-ui
   "jirc: $Id: test.scm,v 1.7 1996/01/29 00:18:32 kbs Exp $"
   default-irc-env))

;; Make a default handler for messages
(define default-msg-handler
  (lambda (msg)
    (frame-append-text
     test-ui
     (concat "[" (irc-msg-command msg) "] " (irc-msg-comma-middle msg)
             (irc-msg-trailer msg) "\n"))))

(irc-set-default-msg-hook! default-irc-env default-msg-handler)


;; Handle text entered by the user
;; One can choose to write a command parser here, but
;; for now, this is now done.

(define user-send
  (lambda (str)
    (irc-send default-irc-env
              (concat "PRIVMSG " (frame-get-current-target test-ui)
                      " :" str "\r\n"))
    (frame-append-text
     test-ui
     (concat "> " str "\n"))))

(frame-set-user-hook! test-ui user-send)

;; Handle certain types of server events
(irc-set-msg-hook!
 default-irc-env "PING"
 (lambda (msg)
   (irc-send default-irc-env "PONG jirc\r\n")))

(irc-set-msg-hook!
 default-irc-env "PRIVMSG"
 (lambda (msg)
   (define target (irc-msg-comma-middle msg))
   (frame-append-text
    test-ui
    (cond
     ((string-starts-with? target "#" 0)
      (concat "<" (irc-msg-prefix-name msg)
              target "> " (irc-msg-trailer msg) "\n"))
     (1
      (concat
       "* "
       (irc-msg-prefix-name msg) "* " (irc-msg-trailer msg) "\n"))))))

;; join/part/quit messages
(irc-set-msg-hook!
 default-irc-env "JOIN"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg) " (" (irc-msg-prefix-host msg)
            ") has joined " (irc-msg-trailer msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "PART"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg)
            " has left " (irc-msg-comma-middle msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "QUIT"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg)
            " quit (" (irc-msg-trailer msg) ")\n"))))

(irc-set-msg-hook!
 default-irc-env "PRIVMSG/ACTION"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg) "->" (irc-msg-comma-middle msg) 
            (irc-msg-ctcp-tag msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "PRIVMSG/VERSION"
 (lambda (msg)
   (irc-send default-irc-env
             (concat
              "NOTICE "
              (irc-msg-prefix-name msg)
              " :\001VERSION jirc:0.1:running $Id: test.scm,v 1.7 1996/01/29 00:18:32 kbs Exp $\001\r\n"))
   (frame-append-text
    test-ui
    (concat "Version query from " (irc-msg-prefix-name msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "PRIVMSG/PING"
 (lambda (msg)
   (irc-send default-irc-env
             (concat
              "NOTICE "
              (irc-msg-prefix-name msg)

              " :\001PING " (irc-msg-ctcp-tag msg) "\001\r\n"))
   (frame-append-text
    test-ui
    (concat "Ping from " (irc-msg-prefix-name msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "PRIVMSG/JIRC-SWOOSH"
 (lambda (msg)
   (context-show-url this (irc-msg-ctcp-tag msg) "_self")
   (frame-append-text
    test-ui
    (concat "Swoosh from " (irc-msg-prefix-name msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "NICK"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg) " is now known as "
            (irc-msg-trailer msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "MODE"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg) " has changed mode "
            (irc-msg-comma-middle msg)
            (irc-msg-trailer msg) "\n"))))

;;
;; Responses from ctcp queries
(irc-set-msg-hook!
 default-irc-env "NOTICE/VERSION"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg) " uses " (irc-msg-ctcp-tag msg) "\n"))))

(irc-set-msg-hook!
 default-irc-env "NOTICE/USERINFO"
 (lambda (msg)
   (frame-append-text
    test-ui
    (concat (irc-msg-prefix-name msg) " is " (irc-msg-ctcp-tag msg) "\n"))))


(frame-show test-ui)
